#
# CanDo - Web based competency tracking system.
# Copyright (C) 2006 Arlington Public Schools, Arlington, VA and The
# Shuttleworth Foundation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Unit tests for cando.generations.evolve10
"""
import unittest

from zope.app.testing import setup
from zope.testing import doctest
from zope.component import provideAdapter

from schooltool.app.interfaces import IPluginInit
from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.app.app import getSchoolToolApplication
from schooltool.schoolyear.interfaces import ISchoolYearContainer
from schooltool.schoolyear.schoolyear import getSchoolYearContainer
from schooltool.group.interfaces import IGroupContainer
from schooltool.group.group import getGroupContainerForApp
from schooltool.basicperson.interfaces import IDemographicsFields
from schooltool.basicperson.demographics import getDemographicsFields
from schooltool.basicperson.demographics import getPersonDemographics
from schooltool.basicperson.demographics import DemographicsInit
from schooltool.basicperson.interfaces import IBasicPerson
from schooltool.basicperson.interfaces import IDemographics
from schooltool.relationship.tests import setUpRelationships

from cando.virginia.interfaces import IStudentDemographics
from cando.virginia.student import StudentDemographics
from cando.virginia.student import CanDoDemographicsInit


def doctest_evolve10():
    """Evolution to generation 10.

        >>> from schooltool.testing.setup import createSchoolToolApplication
        >>> from zope.app.component.site import LocalSiteManager
        >>> from zope.app.component.hooks import setSite
        >>> from cando.generations.tests import ContextStub
        >>> context = ContextStub()
        >>> context.root_folder['app'] = app  = createSchoolToolApplication()
        >>> app.setSiteManager(LocalSiteManager(app))
        >>> setSite(app)

    Let's add some persons:

        >>> from schooltool.basicperson.person import BasicPerson
        >>> persons = app['persons']
        >>> persons['will'] = BasicPerson("will", "William", "Straus")
        >>> persons['vlad'] = BasicPerson("vlad", "Vladimir", "Rubov")
        >>> persons['john'] = BasicPerson("john", "Johny", "John")
        >>> persons['pete'] = BasicPerson("pete", "Petey", "Pete")
        >>> persons['bill'] = BasicPerson("bill", "Billy", "Bill")
        >>> persons['marcela'] = BasicPerson("marcela", "Marcela", "Ortiz")
        >>> persons['camila'] = BasicPerson("camila", "Camila", "Cerna")

    And set CanDo demographics on them:

        >>> will_demographics = IStudentDemographics(persons['will'])
        >>> vlad_demographics = IStudentDemographics(persons['vlad'])
        >>> john_demographics = IStudentDemographics(persons['john'])
        >>> pete_demographics = IStudentDemographics(persons['pete'])
        >>> bill_demographics = IStudentDemographics(persons['bill'])
        >>> marcela_demographics = IStudentDemographics(persons['marcela'])
        >>> camila_demographics = IStudentDemographics(persons['camila'])

        >>> will_demographics.ethnic_code = 'a'
        >>> will_demographics.state_report_id = 'will_id'
        >>> will_demographics.ethnicity
        u'Asian'
                
        >>> vlad_demographics.ethnic_code = 'i'
        >>> vlad_demographics.state_report_id = 'vlad_id'
        >>> vlad_demographics.ethnicity
        u'American Indian'

        >>> john_demographics.ethnic_code = 'b'
        >>> john_demographics.state_report_id = 'john_id'
        >>> john_demographics.ethnicity
        u'Black'
        
        >>> pete_demographics.ethnic_code = 'w'
        >>> pete_demographics.state_report_id = 'pete_id'
        >>> pete_demographics.ethnicity
        u'White'

        >>> bill_demographics.ethnic_code = 'u'
        >>> bill_demographics.state_report_id = 'bill_id'
        >>> bill_demographics.ethnicity
        u'Unspecified'
        >>> bill_demographics.grade_class = u'10'

        >>> marcela_demographics.ethnic_code = 'h'   
        >>> marcela_demographics.state_report_id = 'marcela_id'
        >>> marcela_demographics.ethnicity
        u'Hispanic'

        >>> camila_demographics.state_report_id = 'camila_id'
        >>> camila_demographics.ethnicity
        u'Unspecified'

    Let's evolve now.

        >>> from cando.generations.evolve10 import evolve
        >>> evolve(context)

    Our application should have SchoolTool demographics fields and
    CanDo demographics fields.

        >>> dfs = IDemographicsFields(app)
        >>> sorted(dfs.keys())
        ['ID', 'citizenship', 'ethnicity', 'grade_class', 'language',
         'placeofbirth', 'state_report_id', 'teacher_id']

    SchoolTool's default ethnicity list is changed.
    
        >>> dfs['ethnicity'].items
        [u'American Indian', u'Asian', u'Black', u'Hispanic', u'Unspecified',
         u'White']

    Now, let's check our new demographics match old CanDo's demographics.

        >>> marcela_demographics = IDemographics(persons['marcela'])
        >>> marcela_demographics['state_report_id']
        u'marcela_id'
        >>> marcela_demographics['ethnicity']
        u'Hispanic'
        >>> marcela_demographics['grade_class'] is None
        True

        >>> bill_demographics = IDemographics(persons['bill'])
        >>> bill_demographics['state_report_id']
        u'bill_id'
        >>> bill_demographics['ethnicity']
        u'Unspecified'
        >>> bill_demographics['grade_class']
        u'10'

        >>> camila_demographics = IDemographics(persons['camila'])
        >>> camila_demographics['state_report_id']
        u'camila_id'
        >>> camila_demographics['ethnicity']
        u'Unspecified'
        >>> camila_demographics['grade_class'] is None
        True

    """


def setUp(test):
    setup.placelessSetUp()
    provideAdapter(getDemographicsFields, [ISchoolToolApplication],
                   IDemographicsFields)
    provideAdapter(getPersonDemographics, [IBasicPerson],
                   IDemographics)
    provideAdapter(StudentDemographics, [IBasicPerson],
                   IStudentDemographics)
    provideAdapter(getSchoolToolApplication, [None],
                   ISchoolToolApplication)
    provideAdapter(getGroupContainerForApp, [ISchoolToolApplication],
                   IGroupContainer)
    provideAdapter(getSchoolYearContainer, [ISchoolToolApplication],
                   ISchoolYearContainer)
    provideAdapter(CanDoDemographicsInit, [ISchoolToolApplication],
                   IPluginInit, name="cando.demographics")
    provideAdapter(DemographicsInit, [ISchoolToolApplication],
                   IPluginInit, name="schooltool.basicperson.demographics")
    setup.setUpAnnotations()
    setup.setUpDependable()
    setup.setUpTraversal()
    setUpRelationships()


def tearDown(test):
    setup.placelessTearDown()


def test_suite():
    optionflags = (doctest.ELLIPSIS |
                   doctest.NORMALIZE_WHITESPACE |
                   doctest.REPORT_ONLY_FIRST_FAILURE)
    return doctest.DocTestSuite(setUp=setUp, tearDown=tearDown,
                                optionflags=optionflags)


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')
