#
# CanDo - Web based competency tracking system.
# Copyright (C) 2006 Arlington Public Schools, Arlington, VA and The
# Shuttleworth Foundation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Virginia Gradebook views.

$Id:$
"""
import datetime
import csv
import tempfile

from zope.component import queryUtility
from zope.traversing.api import getParents
from zope.security.proxy import removeSecurityProxy

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.course.interfaces import ISection, ISectionContainer
from schooltool.term.interfaces import IDateManager
from schooltool.basicperson.interfaces import IDemographics

from cando.gradebook.gradebook import _getEvaluation
from cando.virginia.interfaces import ICompetencyReference
from cando.virginia.competency import getCoreCompetencies
from cando.virginia.competency import buildCompetencyDict


def getFlatCompetencies(rootComp):
    """This builds a flat list of a competency tree.

    It only includes competencies and not competency groups.
    """
    if ICompetencyReference.providedBy(rootComp):
        competencies = [rootComp]
    else:
        competencies = []
        if hasattr(rootComp, 'values'):
            for comp in rootComp.values():
                competencies += getFlatCompetencies(comp)
        else:
            competencies += rootComp
    return competencies


def formatPercent(value):
    return '%2.0f%%' % (value + 0.0000000001)


class BaseReportView(object):
    def __init__(self, context, request):
        self.context = removeSecurityProxy(context)
        self.request = request
        self.scoresystem = ISchoolToolApplication(None)['scoresystem']
        for obj in reversed(getParents(context)):
            if ISection.providedBy(obj):
                self.section = obj
                break
        self.comps = getFlatCompetencies(self.context.comprefDict)

        # build dictionary of evaluations by student
        self.byStudent = {}
        for student in self.section.members:
            for comp in self.comps:
                ev = self.context.getEvaluation(student, comp)
                if ev is None or \
                   self.scoresystem.getNumericalValue(ev.value) is None:
                    continue
                self.byStudent.setdefault(student, {})[comp] = ev.value.value

        # aggregate student evaluations into aggregate dictionary
        self.studentAggregate = {}
        for student, compDict in self.byStudent.items():
            aggregate = self.studentAggregate.setdefault(student, {})
            numCompetent = totalPoints = 0
            for value in compDict.values():
                totalPoints += value
                if value >= self.scoresystem.minPassingScore.value:
                    numCompetent += 1
            aggregate['numGraded'] = len(compDict)
            aggregate['totalPoints'] = totalPoints
            aggregate['numCompetent'] = numCompetent
            aggregate['percentCompetent'] = float(100 * numCompetent) / \
                float(len(compDict))
            aggregate['numCompsTimes3'] = float(100 * totalPoints) / \
                float(3 * len(compDict))

        # finally, loop through aggregates, summarizing values
        self.numStudents = len(self.byStudent)
        if len(list(self.section.members)):
            self.percentTotalStudents = float(100 * self.numStudents) / \
                float(len(list(self.section.members)))
        else:
            self.percentTotalStudents = 0.0
        self.averageNumberCompetencies = 0
        self.averagePercentCompetent = 0.0
        self.averageGrade = 0.0
        self.averagePoints = 0.0
        self.num90Competent = self.num80Competent = self.num70Competent = \
            self.num60Competent = self.num100Grade = self.num90Grade = \
            self.num80Grade = self.num70Grade = self.num60Grade = \
            self.numLess60Grade = 0
        for aggregate in self.studentAggregate.values():
            self.averageNumberCompetencies += aggregate['numCompetent']
            self.averagePercentCompetent += aggregate['percentCompetent']
            self.averageGrade += aggregate['numCompsTimes3']
            self.averagePoints += aggregate['totalPoints']
            if aggregate['percentCompetent'] >= 90.0:
                self.num90Competent += 1
            if aggregate['percentCompetent'] >= 80.0:
                self.num80Competent += 1
            if aggregate['percentCompetent'] >= 70.0:
                self.num70Competent += 1
            if aggregate['percentCompetent'] >= 60.0:
                self.num60Competent += 1
            if aggregate['numCompsTimes3'] >= 100.0:
                self.num100Grade += 1
            if aggregate['numCompsTimes3'] >= 90.0:
                self.num90Grade += 1
            if aggregate['numCompsTimes3'] >= 80.0:
                self.num80Grade += 1
            if aggregate['numCompsTimes3'] >= 70.0:
                self.num70Grade += 1
            if aggregate['numCompsTimes3'] >= 60.0:
                self.num60Grade += 1
            if aggregate['numCompsTimes3'] < 60.0:
                self.numLess60Grade += 1
        if self.numStudents:
            self.averageNumberCompetencies = self.averageNumberCompetencies / \
                self.numStudents
            self.averagePercentCompetent = self.averagePercentCompetent / \
                float(self.numStudents)
            self.averageGrade = self.averageGrade / \
                float(self.numStudents)
            self.averagePoints = self.averagePoints / \
                float(self.numStudents)
        self.percent90Competent = self.calcPercent(self.num90Competent)
        self.percent80Competent = self.calcPercent(self.num80Competent)
        self.percent70Competent = self.calcPercent(self.num70Competent)
        self.percent60Competent = self.calcPercent(self.num60Competent)
        self.percent100Grade = self.calcPercent(self.num100Grade)
        self.percent90Grade = self.calcPercent(self.num90Grade)
        self.percent80Grade = self.calcPercent(self.num80Grade)
        self.percent70Grade = self.calcPercent(self.num70Grade)
        self.percent60Grade = self.calcPercent(self.num60Grade)
        self.percentLess60Grade = self.calcPercent(self.numLess60Grade)

    def calcPercent(self, value):
        if self.numStudents:
            return float(100 * value) / float(self.numStudents)
        else:
            return 0.0
    def printDict(self):
        for k, v in self.byStudent.items():
            print k.title
            for comp in v:
                print '    %s: %s' % (comp.id, str(self.byStudent[k][comp]))


class VirginiaReportView(BaseReportView):
    def getStudents(self):
        results = []
        for student in sorted(self.studentAggregate, key=lambda x: x.title):
            aggregate = self.studentAggregate[student]
            dict = {'title': student.title,
                    'percentCompetent':
                        formatPercent(aggregate['percentCompetent']),
                    'numCompetent': aggregate['numCompetent'],
                    'numGraded': aggregate['numGraded']}
            results.append(dict)
        return results

    def getAggregate(self):
        return {'numStudents': self.numStudents,
                'percentTotalStudents':
                    formatPercent(self.percentTotalStudents),
                'num90Competent': self.num90Competent,
                'percent90Competent': formatPercent(self.percent90Competent),
                'num80Competent': self.num80Competent,
                'percent80Competent': formatPercent(self.percent80Competent),
                'num70Competent': self.num70Competent,
                'percent70Competent': formatPercent(self.percent70Competent),
                'num60Competent': self.num60Competent,
                'percent60Competent': formatPercent(self.percent60Competent),
                'averagePercentCompetent':
                    formatPercent(self.averagePercentCompetent),
                'averageNumberCompetencies': self.averageNumberCompetencies}


class SectionReportView(object):

    def __init__(self, context, request):
        self.context = removeSecurityProxy(context)
        self.request = request
        self.scoresystem = ISchoolToolApplication(None)['scoresystem']
        for obj in reversed(getParents(context)):
            if ISection.providedBy(obj):
                self.section = removeSecurityProxy(obj)
                break
        comps = getFlatCompetencies(self.context.comprefDict)
        self.all_total = len(comps)

        required_comps = [comp
                          for comp in self.getCoreCompetencies(self.section)
                          if comp.optionalFlag == "required"]
        self.required_total = len(required_comps)

        rows = []
        attainment_labels = {100: "90-100%",
                             90: "80-89%",
                             80: "70-79%",
                             70: "60-69%",
                             60: "<= 59%"}
        rows_attainment = {100: 0,
                           90: 0,
                           80: 0,
                           70: 0,
                           60: 0}
        rows_attainment80 = {u"80% or above": 0,
                             u"Below 80%": 0}

        self.student_total = len(self.section.members)

        for student in self.section.members:
            demographics = IDemographics(student)
            info = {"first_name": student.first_name,
                    "last_name": student.last_name,
                    "state_report_id": demographics.get("state_report_id", ""),
                    "required_evaluated": 0,
                    "required_competent": 0,
                    "all_evaluated": 0,
                    "all_competent": 0}
            for comp in comps:
                ev = self.context.getEvaluation(student, comp)
                if ev is None or \
                   self.scoresystem.getNumericalValue(ev.value) is None:
                    continue
                else:
                    info["all_evaluated"] += 1
                    if comp in required_comps:
                        info["required_evaluated"] += 1
                    value = ev.value.value
                    # XXX: Hard coded 2 asked by Welsh
                    if value >= 2: 
                        info["all_competent"] += 1
                        if comp in required_comps:
                            info["required_competent"] += 1
            try:
                required_rating = float(info["required_competent"] * 100) / \
                                  float(info["required_evaluated"])
            except (ZeroDivisionError,):
                required_rating = 0.0
            info["required_rating"] = formatPercent(required_rating)
            try:
                required_completed = float(info["required_competent"] * 100) / \
                                     float(self.required_total)
            except (ZeroDivisionError,):
                required_completed = 0.0
            info["required_completed"] = formatPercent(required_completed)
            try:
                all_rating = float(info["all_competent"] * 100) / \
                             float(info["all_evaluated"])
            except (ZeroDivisionError,):
                all_rating = 0.0

            if required_completed >= 90:
                rows_attainment[100] += 1
            elif required_completed >= 80:
                rows_attainment[90] += 1
            elif required_completed >= 70:
                rows_attainment[80] += 1
            elif required_completed >= 60:
                rows_attainment[70] += 1
            else:
                rows_attainment[60] += 1

            if required_completed >= 80:
                rows_attainment80[u"80% or above"] += 1
            else:
                rows_attainment80[u"Below 80%"] += 1
                
            info["all_rating"] = formatPercent(all_rating)
            rows.append(info)
        self.rows = sorted(rows,
                           key=lambda x:(x["last_name"], x["first_name"]))
        self.rows_attainment = []
        self.rows_attainment80 = []
        for key in sorted(rows_attainment, reverse=True):
            info = {"scale": attainment_labels[key],
                    "student_count": rows_attainment[key]}
            try:
                student_percent = "%.2f" % (float(rows_attainment[key] * 100) / float(self.student_total),)
            except (ZeroDivisionError,):
                student_percent = "0.00"
            info["student_percent"] = student_percent
            self.rows_attainment.append(info)
        for key in sorted(rows_attainment80):
            info = {"scale": key,
                    "student_count": rows_attainment80[key]}
            try:
                student_percent = "%.2f" % (float(rows_attainment80[key] * 100) / float(self.student_total),)
            except (ZeroDivisionError,):
                student_percent = "0.00"
            info["student_percent"] = student_percent
            self.rows_attainment80.append(info)

        self.courses = list(self.section.courses)
        self.section_local_courses = []
        title = u"No section description"
        section_code = ", ".join([course.course_id
                                  for course in self.courses
                                  if course.course_id is not None])
        self.section_local_courses = self.courses
        if self.section.description:
            if len(self.section.description.split(",")) == 2:
                title = self.section.description.split(",")[0].strip()
                section_code = self.section.description.split(",")[1].strip()
                self.section_local_courses = []
                for course in self.courses:
                    if section_code in course.course_id:
                        self.section_local_courses.append(course)
            else:
                title = self.section.description
        self.section_title = title
        self.section_code = section_code
        self.instructors = u", ".join(["%s %s" % (i.first_name, i.last_name)
                                       for i in self.section.instructors])

    def getCoreCompetencies(self, section):
        result = {}
        for comp in getCoreCompetencies(section):
            result[comp] = None
        for course in list(section.courses):
            for comp in getCoreCompetencies(course):
                result[comp] = None
        return result.keys()
 
 
class TeacherReportView(BaseReportView):
    def getStudents(self):
        results = []
        for student in sorted(self.studentAggregate, key=lambda x: x.title):
            aggregate = self.studentAggregate[student]
            dict = {'title': student.title,
                    'totalPoints': aggregate['totalPoints'],
                    'percentCompetent':
                        formatPercent(aggregate['percentCompetent']),
                    'ratedGreater3': aggregate['numCompetent'],
                    'evaluated': aggregate['numGraded'],
                    'evaluatedTimes3': 3 * aggregate['numGraded'],
                    'numCompsTimes3':
                        formatPercent(aggregate['numCompsTimes3'])}
            results.append(dict)
        return results

    def getAggregate(self):
        return {'averageGrade': formatPercent(self.averageGrade),
                'averagePoints': '%2.2f' % self.averagePoints,
                'totalStudents': self.numStudents,
                'num100Grade': self.num100Grade,
                'percent100Grade': formatPercent(self.percent100Grade),
                'num90Grade': self.num90Grade,
                'percent90Grade': formatPercent(self.percent90Grade),
                'num80Grade': self.num80Grade,
                'percent80Grade': formatPercent(self.percent80Grade),
                'num70Grade': self.num70Grade,
                'percent70Grade': formatPercent(self.percent70Grade),
                'num60Grade': self.num60Grade,
                'percent60Grade': formatPercent(self.percent60Grade),
                'numLess60Grade': self.numLess60Grade,
                'percentLess60Grade': formatPercent(self.percentLess60Grade)}


class AllSectionsReportView(object):
    def __init__(self, context, request):
        self.context = removeSecurityProxy(context)
        self.request = request
        self.scoresystem = ISchoolToolApplication(None)['scoresystem']
        self.rows = []
        self.date = datetime.datetime.today().strftime("%b %d, %Y %I:%M%p")
        term = queryUtility(IDateManager).current_term
        for section in ISectionContainer(term).values():

            # get comps for the section
            comprefDict = buildCompetencyDict(section)
            comps = getFlatCompetencies(comprefDict)

            # loop though comps, students, aggregating values
            total = len(comps)
            byStudent = {}
            for comp in comps:
                for student in section.members:
                    ev = _getEvaluation(student, comp)
                    if ev is None or \
                       self.scoresystem.getNumericalValue(ev.value) is None:
                        continue
                    byStudent.setdefault(student, {})[comp] = ev.value.value

            # aggregate student evaluations into aggregate dictionary
            studentAggregate = {}
            for student, compDict in byStudent.items():
                aggregate = studentAggregate.setdefault(student, {})
                numCompetent = 0
                for value in compDict.values():
                    if value >= self.scoresystem.minPassingScore.value:
                        numCompetent += 1
                aggregate['numGraded'] = len(compDict)
                aggregate['percentCompetent'] = float(100 * numCompetent) / \
                    float(len(compDict))
                aggregate['percentAllCompetent'] = float(100 * numCompetent) / \
                    float(total)

            # finally, loop through aggregates, summarizing values
            numStudents = len(studentAggregate)
            averageNumberGraded = 0
            averagePercentCompetent = 0.0
            averagePercentAllCompetent = 0.0
            average80Competent = 0.0
            for aggregate in studentAggregate.values():
                averageNumberGraded += aggregate['numGraded']
                averagePercentCompetent += aggregate['percentCompetent']
                averagePercentAllCompetent += aggregate['percentAllCompetent']
                if aggregate['percentCompetent'] >= 80.0:
                    average80Competent += 1
            if numStudents:
                averageNumberGraded = averageNumberGraded / \
                    numStudents
                averagePercentCompetent = averagePercentCompetent / \
                    float(numStudents)
                averagePercentAllCompetent = averagePercentAllCompetent / \
                    float(numStudents)
                average80Competent = float(100 * average80Competent) / \
                    float(numStudents)

            # build and add row to rows
            row = {
                "division": self.getDivisionsTitles(section),
                "course": self.getCoursesTitles(section),
                "section": section.title,
                "instructor": self.getInstructorsNames(section),
                "last_name": self.getFirstInstructorLastName(section),
                "total": total,
                "evaluated": averageNumberGraded,
                "all_competent": formatPercent(averagePercentAllCompetent).strip(),
                "marked_competent": formatPercent(averagePercentCompetent).strip(),
                "marked_competent80": formatPercent(average80Competent).strip(),
                }
            self.rows.append(row)

        # sort rows by division unless Instructor is specified in the request
        sortingKey = lambda x:(x['division'],x['course'],x['last_name'])
        self.sortBy = u"Program Area"
        if self.request.get("sortBy") == 'Instructor':
            sortingKey = lambda x:(x['last_name'],x['division'],x['course'])
            self.sortBy = u"Instructor"
        self.rows = sorted(self.rows, key=sortingKey)

    def getDivision(self, course):
        comps = getCoreCompetencies(course)
        try:
            return comps[0].__parent__.__parent__.__parent__.title
        except (IndexError, AttributeError,):
            return ''

    def getInstructorsNames(self, section):
        return ", ".join(["%s %s" % (instructor.first_name,
                                     instructor.last_name,)
                          for instructor in section.instructors])

    def getCoursesTitles(self, section):
        return ", ".join([course.title for course in section.courses])

    def getDivisionsTitles(self, section):
        return ", ".join([self.getDivision(course)
                          for course in section.courses
                          if self.getDivision(course)])

    def getFirstInstructorLastName(self, section):
        try:
            first_instructor = list(section.instructors)[0]
            return first_instructor.last_name
        except (IndexError, AttributeError,):
            return ""


# XXX: use baseclass
class AllSectionsRequiredReportView(object):
    def __init__(self, context, request):
        self.context = removeSecurityProxy(context)
        self.request = request
        self.scoresystem = ISchoolToolApplication(None)['scoresystem']
        self.rows = []
        self.date = datetime.datetime.today().strftime("%b %d, %Y %I:%M%p")
        term = queryUtility(IDateManager).current_term
        for section in ISectionContainer(term).values():

            # get comps for the section
            comps = self.getCoreCompetencies(section)

            # loop though comps, students, aggregating values
            required = 0
            byStudent = {}
            for comp in comps:
                if comp.optionalFlag == "required":
                    required += 1
                    for student in section.members:
                        ev = _getEvaluation(student, comp)
                        if ev is None or \
                           self.scoresystem.getNumericalValue(ev.value) is None:
                            continue
                        byStudent.setdefault(student, {})[comp] = ev.value.value

            # aggregate student evaluations into aggregate dictionary
            studentAggregate = {}
            for student, compDict in byStudent.items():
                aggregate = studentAggregate.setdefault(student, {})
                numCompetent = 0
                for value in compDict.values():
                    if value >= self.scoresystem.minPassingScore.value:
                        numCompetent += 1
                aggregate['numGraded'] = len(compDict)
                aggregate['percentCompetent'] = float(100 * numCompetent) / \
                    float(len(compDict))
                aggregate['percentAllCompetent'] = float(100 * numCompetent) / \
                    float(required)

            # finally, loop through aggregates, summarizing values
            numStudents = len(studentAggregate)
            averageNumberGraded = 0
            averagePercentCompetent = 0.0
            averagePercentAllCompetent = 0.0
            average80Competent = 0.0
            for aggregate in studentAggregate.values():
                averageNumberGraded += aggregate['numGraded']
                averagePercentCompetent += aggregate['percentCompetent']
                averagePercentAllCompetent += aggregate['percentAllCompetent']
                if aggregate['percentAllCompetent'] >= 80.0:
                    average80Competent += 1
            if numStudents:
                averageNumberGraded = averageNumberGraded / \
                    numStudents
                averagePercentCompetent = averagePercentCompetent / \
                    float(numStudents)
                averagePercentAllCompetent = averagePercentAllCompetent / \
                    float(numStudents)
                average80Competent = float(100 * average80Competent) / \
                    float(numStudents)

            # build and add row to rows
            row = {
                "division": self.getDivisionsTitles(section),
                "course": self.getCoursesTitles(section),
                "section": section.title,
                "instructor": self.getInstructorsNames(section),
                "last_name": self.getFirstInstructorLastName(section),
                "total": required,
                "evaluated": averageNumberGraded,
                "all_competent": formatPercent(averagePercentAllCompetent).strip(),
                "marked_competent": formatPercent(averagePercentCompetent).strip(),
                "marked_competent80": formatPercent(average80Competent).strip(),
                }
            self.rows.append(row)

        # sort rows by division unless Instructor is specified in the request
        sortingKey = lambda x:(x['division'],x['course'],x['last_name'])
        self.sortBy = u"Program Area"
        if self.request.get("sortBy") == 'Instructor':
            sortingKey = lambda x:(x['last_name'],x['division'],x['course'])
            self.sortBy = u"Instructor"
        self.rows = sorted(self.rows, key=sortingKey)

    def getDivision(self, course):
        comps = getCoreCompetencies(course)
        try:
            return comps[0].__parent__.__parent__.__parent__.title
        except (IndexError, AttributeError,):
            return ''

    def getInstructorsNames(self, section):
        return ", ".join(["%s %s" % (instructor.first_name,
                                     instructor.last_name,)
                          for instructor in section.instructors])

    def getCoursesTitles(self, section):
        return ", ".join([course.title for course in section.courses])

    def getDivisionsTitles(self, section):
        return ", ".join([self.getDivision(course)
                          for course in section.courses
                          if self.getDivision(course)])

    def getFirstInstructorLastName(self, section):
        try:
            first_instructor = list(section.instructors)[0]
            return first_instructor.last_name
        except (IndexError, AttributeError,):
            return ""

    def getCoreCompetencies(self, section):
        result = {}
        for comp in getCoreCompetencies(section):
            result[comp] = None
        for course in list(section.courses):
            for comp in getCoreCompetencies(course):
                result[comp] = None
        return result.keys()


class VAExportView(object):
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.scoresystem = ISchoolToolApplication(None)['scoresystem']
        self.rows = []
        self.date = datetime.datetime.today().strftime("%b %d, %Y %I:%M%p")
        term = queryUtility(IDateManager).current_term
        year = term.last.year
        for section in ISectionContainer(term).values():

            # get core comps for the section
            core_required_comps = [comp
                                   for comp in self.getCoreCompetencies(section)
                                   if comp.optionalFlag == "required"]

            num_core_required_comps = len(core_required_comps)

            state_course_code = self.getCoursesStateCodes(section)
            local_course_code = self.getCoursesLocalCodes(section)
            
            comprefDict = buildCompetencyDict(section)
            comps = getFlatCompetencies(comprefDict)

            # loop though comps, students, aggregating values
            for student in section.members:
                demographics = IDemographics(student)
                testing_number = demographics.get('state_report_id', '')
                row = [testing_number, student.username,
                       state_course_code, local_course_code,
                       num_core_required_comps]
                numCoreRequiredCompetent = 0
                numGraded = 0
                numCompetent = 0
                for comp in comps:
                    ev = _getEvaluation(student, comp)
                    if ev is None or \
                       self.scoresystem.getNumericalValue(ev.value) is None:
                        continue
                    value = ev.value.value
                    numGraded += 1
                    if value >= self.scoresystem.minPassingScore.value:
                        numCompetent += 1
                        if comp in core_required_comps:
                            numCoreRequiredCompetent += 1
                row.append(numCoreRequiredCompetent)
                self.rows.append(row)
        self.rows = sorted(self.rows)

    def __call__(self):
        result = tempfile.TemporaryFile()
        writer = csv.writer(result, dialect=csv.excel_tab)
        writer.writerows(self.rows)
        self.request.response.setHeader("Content-Type", "text/csv")
        return result

    def getDivision(self, course):
        comps = getCoreCompetencies(course)
        try:
            return comps[0].__parent__.__parent__.__parent__.title
        except (IndexError, AttributeError,):
            return ''

    def getInstructorsNames(self, section):
        return ", ".join(["%s %s" % (instructor.first_name,
                                     instructor.last_name,)
                          for instructor in section.instructors])

    def getCoursesTitles(self, section):
        return ", ".join([course.title for course in section.courses])

    def getCoursesStateCodes(self, section):
        return ", ".join([course.government_id for course in section.courses
                          if course.government_id is not None])

    def getCoursesLocalCodes(self, section):
        courses = list(section.courses)
        section_code = ", ".join([course.course_id
                                  for course in courses
                                  if course.course_id is not None])
        if section.description:
            if len(section.description.split(",")) == 2:
                section_code = section.description.split(",")[1].strip()
        return section_code

    def getDivisionsTitles(self, section):
        return ", ".join([self.getDivision(course)
                          for course in section.courses
                          if self.getDivision(course)])

    def getFirstInstructorLastName(self, section):
        try:
            first_instructor = list(section.instructors)[0]
            return first_instructor.last_name
        except (IndexError, AttributeError,):
            return ""

    def getCoreCompetencies(self, section):
        result = {}
        for comp in getCoreCompetencies(section):
            result[comp] = None
        for course in list(section.courses):
            for comp in getCoreCompetencies(course):
                result[comp] = None
        return result.keys()
