#
# CanDo - Web based competency tracking system.
# Copyright (C) 2006 Arlington Public Schools, Arlington, VA and The
# Shuttleworth Foundation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
from zope.app import zapi
from zope.publisher.browser import BrowserView
from zope.security.proxy import removeSecurityProxy
from zope.security.checker import canWrite
from zope.traversing.api import getParents
from zope.formlib import form
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
from zope.app.container.interfaces import INameChooser
from zope.event import notify
from zope.lifecycleevent import ObjectCreatedEvent

from schooltool.app.browser import app
from schooltool.skin.containers import ContainerView
from zope.dublincore.interfaces import IZopeDublinCore
from schooltool.table.batch import IterableBatch
from cando.message.interfaces import IMessageBox, IMessage
from cando.message.interfaces import IForum, IQueueItem
from cando.message.message import URIMessageBox, locateMessageBox, Message
from schooltool.person.interfaces import IPerson
from schooltool.app.browser import ViewPreferences
from schooltool.app.app import getSchoolToolApplication
from schooltool.dashboard.dashboard import PersonDashboardCategory
from schooltool.relationship import getRelatedObjects
from schooltool.app import relationships
from schooltool.course.interfaces import ISection
from schooltool.securitypolicy.crowds import ManagersCrowd, TeachersCrowd

from pytz import utc
from zope.i18nmessageid import MessageFactory

_ = MessageFactory('cando')


class PersonMessageBoxesProxyView(object):
    """View for the listing of a person's section messageboxes."""

    def update(self):
        person = self.context.person
        related = getRelatedObjects(person, relationships.URIGroup)
        self.sections = filter(ISection.providedBy, related)


class SectionMessageBoxesProxyView(object):
    """View for the listing of a person's section messageboxes."""

    def update(self):
        section = self.context.section
        related = getRelatedObjects(section, relationships.URIMember)
        self.persons = filter(IPerson.providedBy, related)


class MessageBoxView(ContainerView):
    """Viewing messageboxes"""

    __used_for__ = IMessageBox

    index_title = _(u"MessageBox index")
    add_title = _(u"Add a new message")
    add_url = "addMessage.html"

    def canModifyMessage(self, message):
        canWrite(message, 'title')

    def canDeleteMessage(self, message):
        return canWrite(message, 'title')

    def sortByDate(self, a, b):
        return cmp(IZopeDublinCore(a).created,
                   IZopeDublinCore(b).created)

    def title(self):
        return self.context.title.replace('MessageBox for', 'Messages for')

    def update(self):
        contents = list(self.context.values())
        contents.sort(self.sortByDate)
        contents.reverse()
        if 'SEARCH' in self.request and 'CLEAR_SEARCH' not in self.request:
            searchstr = self.request['SEARCH'].lower()
            results = [item for item in contents
                       if searchstr in item.title.lower()]
        else:
            self.request.form['SEARCH'] = ''
            results = contents
        
        self.results = results
        self.batch = IterableBatch(results, self.request)

    def getSection(self):
        for obj in getParents(self.context):
            if ISection.providedBy(obj):
                return obj

    @property
    def isTeacherOrManager(self):
        principal = self.request.principal
        person = IPerson(principal)
        section = self.getSection()
        messageBox = self.context
        if person in messageBox.managers:
            return True
        if TeachersCrowd(messageBox).contains(principal):
            return True
        if ManagersCrowd(messageBox).contains(principal):
            return True
        if section and person in section.instructors:
            return True
        return False



class MessageThreadedView(ContainerView):
    """Viewing messageboxes"""

    __used_for__ = IMessage

    def __init__(self, context, request):
        self.context = removeSecurityProxy(context)
        self.request = request
        self.update()

    def getDeleteAction(self):
        parentURL = zapi.absoluteURL(self.context.__parent__, self.request)
        return parentURL + '/delete.html'

    def getReturnURL(self):
        messageBox = locateMessageBox(self.context)
        returnUrl = zapi.absoluteURL(messageBox, self.request)
        return returnUrl

    def canDeleteMessage(self):
        return canWrite(self.context, 'title')

    def formatDate(self, date):
        return ViewPreferences(self.request).renderDatetime(utc.localize(date))

    def isForum(self):
        messageBox = locateMessageBox(self.context)
        return IForum.providedBy(messageBox)

    def teacherIsActive(self):
        messageBox = locateMessageBox(self.context)
        self.queueItem = IQueueItem(self.context)
        if self.queueItem.status is None:
            return False
        if self.isTeacher() and 'pending' in self.queueItem.status:
            return True
        return False

    def studentIsActive(self):
        return not self.isTeacher() and bool(self.getStatus())

    def studentHasButton(self):
        if self.isTeacher():
            return False
        messageBox = locateMessageBox(self.context)
        self.queueItem = IQueueItem(self.context)
        if self.queueItem.status is None:
            return 'Submit for Review'
        if 'processed' in self.queueItem.status:
            return 'Resubmit for Review'
        if 'pending' in self.queueItem.status:
            return False

    def getStatus(self):
        messageBox = locateMessageBox(self.context)
        self.queueItem = IQueueItem(self.context)
        if self.queueItem.status is None:
            return False
        elif 'processed' in self.queueItem.status:
            return 'Already Reviewed'
        elif 'pending' in self.queueItem.status:
            return 'Pending Review'

    def update(self):
        if self.context.__name__ in self.request:
            if "private_response" not in self.request:
                self.queueItem = IQueueItem(self.context)
                self.queueItem.queue()
            else:
                url = zapi.absoluteURL(self.context, self.request) + \
                      "/+/private.html"
                self.request.response.redirect(url)
        elif 'DELETE' in self.request:
            if self.canDeleteMessage():
                return True

    def getSection(self):
        for obj in getParents(self.context.__parent__):
            if ISection.providedBy(obj):
                return obj

    def teacherMessage(self):
        principal = self.request.principal
        person = IPerson(principal)
        #import pdb;pdb.set_trace()
        if person.title == self.context.author.title:
            return False
        else:
            return True

    def isAuthor(self):
        principal = self.request.principal
        person = IPerson(principal)
        return person == self.context.author

    def canRespondPrivately(self):
        if self.isForum() and not self.isAuthor():
            if not self.isTeacher() and self.ownerIsTeacher():
                return True
            if self.isTeacher() and not self.ownerIsTeacher():
                return True
        return False

    def ownerIsTeacher(self):
        person = self.context.author
        section = self.getSection()
        messageBox = locateMessageBox(self.context)
        if person in messageBox.managers:
            return True
        if TeachersCrowd(messageBox).contains(person):
            return True
        if ManagersCrowd(messageBox).contains(person):
            return True
        if section and person in section.instructors:
            return True
        return False

    def isTeacher(self, person=None):
        principal = self.request.principal
        if person is None:
            person = IPerson(principal)
        section = self.getSection()
        messageBox = locateMessageBox(self.context)
        if person in messageBox.managers:
            return True
        if TeachersCrowd(messageBox).contains(principal):
            return True
        if ManagersCrowd(messageBox).contains(principal):
            return True
        if section and person in section.instructors:
            return True
        return False


class MessageAddView(app.BaseAddView):
    """Adding messagebox entries"""
    def create(self, *args, **kw):
        """Do the actual instantiation."""
        message = self._factory(*args, **kw)
        message.author = removeSecurityProxy(IPerson(self.request.principal))
        return message

    def createAndAdd(self, data):
        content = super(MessageAddView, self).createAndAdd(data)
        if not self.isTeacher():
            queueItem = removeSecurityProxy(IQueueItem(content))
            queueItem.queue()
        return content

    def isTeacher(self, person=None):
        principal = self.request.principal
        if person is None:
            person = IPerson(principal)
        section = self.getSection()
        messageBox = locateMessageBox(self.context)
        if person in messageBox.managers:
            return True
        if TeachersCrowd(messageBox).contains(principal):
            return True
        if ManagersCrowd(messageBox).contains(principal):
            return True
        if section and person in section.instructors:
            return True
        return False

    def getSection(self):
        for obj in getParents(self.context.__parent__):
            if ISection.providedBy(obj):
                return obj


class MessageEditView(app.BaseEditView):
    """Editing messagebox entries"""


class MessageDeleteView(BrowserView):
    """A view for deleting items from container."""

    def getTitle(self):
        message = self.context.get(str(self.request['messageName']), None)
        if message is not None:
            return message.__name__

    def update(self):
        if 'CONFIRM' in self.request:
            del self.context[unicode(self.request['messageName'])]
            self.request.response.redirect(self.request['returnURL'])
        elif 'CANCEL' in self.request:
            self.request.response.redirect(self.request['returnURL'])


class ForumDeleteView(BrowserView):
    """A view for deleting items of the section forum."""

    def update(self):
        if 'CONFIRM' in self.request:
            names = [v.__name__ for v in self.context.values()
                     if IMessage.providedBy(v)]
            for name in names:
                del self.context[name]
            self.request.response.redirect(self.getReturnURL())
        elif 'CANCEL' in self.request:
            self.request.response.redirect(self.getReturnURL())

    def getReturnURL(self):
        section = removeSecurityProxy(self.context).section
        return zapi.absoluteURL(section, self.request) + '/forum'


class MessageBoxMemberView(app.RelationshipViewBase):
    """View for managing messagebox members"""

    title = _(u"MessageBox Members")
    current_title = _(u"Current Members of this MessageBox")
    available_title = _(u"Available Members to add")

    def getCollection(self):
        return self.context.members

    def getAvailableItems(self):
        container = getSchoolToolApplication()['persons']
        selected_items = set(self.getSelectedItems())
        return [p for p in container.values()
                if p not in selected_items]


class MessageBoxManagerView(app.RelationshipViewBase):
    """View for managing messagebox managers"""

    title = _(u"MessageBox Managers")
    current_title = _(u"Current Managers of this MessageBox")
    available_title = _(u"Available Managers to add")

    def getCollection(self):
        return self.context.managers

    def getAvailableItems(self):
        container = getSchoolToolApplication()['persons']
        selected_items = set(self.getSelectedItems())
        return [p for p in container.values()
                if p not in selected_items]

    def getAvailableItemsContainer(self):
        return getSchoolToolApplication()['persons']


class MessageBoxViewlet(PersonDashboardCategory):
    """A viewlet showing data dealing with messageboxes"""

    def update(self):
        person = self.context #PersonDashboardCategory handles this
        sections = filter(ISection.providedBy, getRelatedObjects(
            person, relationships.URIGroup))
        allMessageBoxes = getSchoolToolApplication()['cando.message.message_box_container']
        self.personalMessageBoxes = {}
        self.sectionMessageBoxes = {}
        self.otherMessageBoxes = set()

        #personal messageboxes
        myMessageBox = allMessageBoxes.getMessageBox(person, None)
        if myMessageBox is not None:
            self.personalMessageBoxes[myMessageBox] = zapi.absoluteURL(
                person, self.request) + '/messagebox/'
        for section in sections:
            messagebox = allMessageBoxes.getMessageBox(person, section)
            #personal section messageboxes
            if messagebox is not None:
                self.personalMessageBoxes[messagebox] = zapi.absoluteURL(
                    person, self.request) + '/messageboxes/' + zapi.name(section)

            #section messageboxes
            messagebox = allMessageBoxes.getMessageBox(None, section)
            if messagebox is not None:
                self.sectionMessageBoxes[messagebox] = zapi.absoluteURL(section, self.request) + \
                                    '/forum/'

        self.otherMessageBoxes = set(getRelatedObjects(person, URIMessageBox))
        self.otherMessageBoxes.difference_update(set(
            self.personalMessageBoxes.keys() + self.sectionMessageBoxes.keys()))
        self.otherMessageBoxes = list(self.otherMessageBoxes)

    def prepareForOutput(self, messageboxes):
        result = []
        for messagebox in messageboxes:
            if self.context in messagebox.members or \
               self.context in messagebox.managers:
                data = {'object': messagebox, 'lastMessage':None,}
                if type(messageboxes) == dict:
                    data['url'] = messageboxes[messagebox]
                entries = list(messagebox.values())
                if entries:
                    entries.sort(lambda a,b: cmp(IZopeDublinCore(a).created,
                                                 IZopeDublinCore(b).created))
                    data['lastMessage'] = entries[-1]
                    data['lastMessageDate'] = IZopeDublinCore(entries[-1]).created
                result.append(data)
        return result

    def getPersonalMessageBoxes(self):
        return self.prepareForOutput(self.personalMessageBoxes)

    def getSectionMessageBoxes(self):
        return self.prepareForOutput(self.sectionMessageBoxes)

    def getOtherMessageBoxes(self):
        return self.prepareForOutput(self.otherMessageBoxes)


class MessageBoxRedirectView(object):
    def __call__(self):
        context = removeSecurityProxy(self.context)
        messagebox = IMessageBox(context)
        url = zapi.absoluteURL(messagebox,self.request)
        self.request.response.redirect(url)


class QueueView(BrowserView):

    def getRootMessage(self, message):
        obj = message
        while obj is not None:
            if IMessageBox.providedBy(obj.__parent__):
                return obj
            obj = obj.__parent__
        raise ValueError("Couldn't find a root message for %r" % repr(message))
            
    def __call__(self):
        roots = []
        for item in self.context.items:
            root_message = self.getRootMessage(item.context)
            if root_message not in roots:
                roots.append(root_message)
        self.root_messages = sorted(roots, key=lambda x:x.created,
                                    reverse=True)
        return super(QueueView, self).__call__()


class PrivateResponseAddView(form.AddForm):

    form_fields = form.Fields(IMessage).select("title", "body")
    label = _(u"Add Private Response")
    template = ViewPageTemplateFile("templates/privateresponse_add.pt")

    @form.action(_(u"Add"), condition=form.haveInputWidgets)
    def addAction(self, action, data):
        self.createAndAdd(data)

    @form.action(_("Cancel"), validator=lambda *x:())
    def handle_cancel_action(self, action, data):
        self.request.response.redirect(self.nextURL())

    def nextURL(self):
        messageBox = locateMessageBox(self.context.context)
        return zapi.absoluteURL(messageBox, self.request)

    def respondedIsFromTeacher(self):
        responded_message = self.context.context
        return self.isTeacher(responded_message.author)

    def add(self, message):
        student = None
        responded_message = removeSecurityProxy(self.context.context)
        if not self.isTeacher():
            if not self.respondedIsFromTeacher():
                self.status = _("Students cannot respond privately to "
                                "other students")
                return
            else:
                student = IPerson(self.request.principal)
        else:
            if self.respondedIsFromTeacher():
                self.status = _("Teachers cannot respond privately to "
                                "other teachers")
                return
            else:
                student = responded_message.author
        section = locateMessageBox(responded_message).section
        messageboxes = locateMessageBox(responded_message).__parent__
        messagebox = messageboxes.getMessageBox(student, section)
        chooser = INameChooser(messagebox)
        copiedmessage = Message(responded_message.title,
                                responded_message.body,
                                responded_message.author)
        name = chooser.chooseName(u"", copiedmessage)
        notify(ObjectCreatedEvent(copiedmessage))
        messagebox[name] = copiedmessage
        chooser = INameChooser(copiedmessage)
        name = chooser.chooseName(u"", message)
        copiedmessage[name] = message
        if not self.isTeacher():
            queueItem = removeSecurityProxy(IQueueItem(message))
            queueItem.queue()
        self._finished_add = True
        return message

    def create(self, data):
        message = Message(data.get("title"), data.get("body"))
        message.author = removeSecurityProxy(IPerson(self.request.principal))
        return message

    def isTeacher(self, person=None):
        principal = self.request.principal
        if person is None:
            person = IPerson(principal)
        section = self.getSection()
        messageBox = locateMessageBox(self.context.context)
        if person in messageBox.managers:
            return True
        if TeachersCrowd(messageBox).contains(person):
            return True
        if ManagersCrowd(messageBox).contains(person):
            return True
        if section and person in section.instructors:
            return True
        return False

    def getSection(self):
        for obj in getParents(self.context.context.__parent__):
            if ISection.providedBy(obj):
                return obj
