#
# CanDo - Web based competency tracking system.
# Copyright (C) 2006 Arlington Public Schools, Arlington, VA and The
# Shuttleworth Foundation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
from datetime import datetime

import persistent.list
from persistent import Persistent
from zope.component import adapts
from zope.interface import implements
from zope.annotation.interfaces import IAnnotations
from zope.app.container.contained import Contained
from zope.app import zapi
from zope.location import location
from zope.security import proxy
from zope.proxy import removeAllProxies

from schooltool.person.interfaces import IPerson
from schooltool.securitypolicy.crowds import Crowd
from schooltool.app.app import getSchoolToolApplication
from schooltool.traverser.traverser import NameTraverserPlugin

from cando.message.interfaces import IMessageBox

from interfaces import IQueue, IQueueItem

QUEUE_KEY = "cando.queue"
QUEUE_ITEM_KEY = "cando.queueItem"


class QueueItem(Persistent, Contained):
    """Queue Item"""
    implements(IQueueItem)
    
    def __init__(self, context, section, person):
        self.context = context
        self.section = section
        self.person = person
        self.history = persistent.list.PersistentList()
        
    @property
    def status(self):
        if len(self.history) > 0:
            return self.history[-1]
        return None
    
    def queue(self):
        if self.status:
            status = self.status[0]
        else:
            status= None
        
        if status == "pending":
            new_status = persistent.list.PersistentList(["processed", datetime.utcnow()])
        if status == "processed" or status is None:
            new_status = persistent.list.PersistentList(["pending", datetime.utcnow()])

        self.history.append(new_status)

 
class SectionQueue(object):
    """Adapt an ``IHaveQueue`` object to ``IQueue``"""
    implements(IQueue)

    def __init__(self, context):
        self.context = context
        self.items = []
        for person in context.members:
            container_key = 'cando.message.message_box_container'
            messagebox_container = getSchoolToolApplication()[container_key]
            messagebox = messagebox_container.getMessageBox(person, context)
            if messagebox is not None:
                self.items.extend(self.findQueuedItems(messagebox))
            
    def findQueuedItems(self, holder):
        queued = []
        for message in holder.values():
            item = IQueueItem(message)
            if item.status and item.status[0] == "pending":
                queued.append(item)
            queued.extend(self.findQueuedItems(message))
        return queued
            
def getQueueItem(context):
    """Adapt an ``IQueueable`` object to ``IQueueItem``"""

    def getMessageBox(message):
        if IMessageBox.providedBy(message):
            return message
        return getMessageBox(zapi.getParent(message))

    annotations = IAnnotations(removeAllProxies(context))
    try:
        queueItem = annotations[QUEUE_ITEM_KEY]
    except KeyError:
        messagebox = getMessageBox(context)
        queueItem = QueueItem(context, messagebox.section, messagebox.person)
        annotations[QUEUE_ITEM_KEY] = queueItem
    #queueItem = location.LocationProxy(queueItem, context, 'queueItem')
    return queueItem
#convention to make adapter introspectable
getQueueItem.factory = QueueItem


class SectionQueueTraverserPlugin(NameTraverserPlugin):
    """Allows traversal to the queue of a section.
    i.e if you go to localhost/sections/somesection/queue this traverser
    will get called instead of a standard traverser. If the queue does not
    exist, it gets created."""

    traversalName = "queue"
    def _traverse(self, request, name):
        context = proxy.removeSecurityProxy(self.context)
        queue = IQueue(self.context)
        #items = IAnnotations(queue)[QUEUE_ITEM_KEY]
        queue = location.LocationProxy(queue, self.context, self.traversalName)
        ###queue.items = items
        return queue

def getQueueSection(context):
    """Adapts an ``IQueue`` to ``ISection``"""
    return context.context #because the IQueue implementations happens to be an adapter whose sections is a context

class QueueItemOwnerCrowd(Crowd):
    """Crowd of the section instructors and the messagebox message owner"""
    adapts(IQueueItem)
    def contains(self, principal):
        return IPerson(principal, None) == self.context.context.author or IPerson(principal, None) in self.context.section.instructors

