#
# CanDo - Web based competency tracking system.
# Copyright (C) 2006 Arlington Public Schools, Arlington, VA and The
# Shuttleworth Foundation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""Skill Driver Views

$Id$
"""
__docformat__ = 'restructuredtest'


from zope.security.proxy import removeSecurityProxy
from zope.session.interfaces import ISession
from zope.viewlet.interfaces import IViewletManager
from zope.app.dependable.interfaces import IDependable

from schooltool.skin.containers import ContainerView
from schooltool.course.interfaces import ICourse
from schooltool.course.course import Course
from schooltool.dashboard.dashboard import PersonDashboardCategory
from schooltool.relationship import getRelatedObjects
from schooltool.app import relationships
from cando.skilldriver.interfaces import ISectionSkillDrivers
from cando.virginia.browser.virginia import SelectedCompetenciesFormView
from cando.virginia.competency import buildCompetencyDict
from cando.virginia.competency import assignAdditionalCompetencies
from cando.virginia.competency import getAdditionalCompetencies

class ISkillDriverViewletManager(IViewletManager):
    """Provides a viewlet hook for the skilldriver."""


class ViewSkillDriver(object):

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.comps = buildCompetencyDict(removeSecurityProxy(self.context))
        self.additionalComps = {}
        self.getSectionAdditionalComps()
        self.getCoursesAdditionalComps()

    def groups(self):
        core_groups = []
        additional_groups = []
        for group in self.comps:
            if self.comps[group]:
                firstComp = self.comps[group][0]
                if self.isAdditionalCompetency(firstComp):
                    additional_groups.append(group)
                else:
                    core_groups.append(group)
        return sorted(core_groups, key=self.sortKey) + \
               sorted(additional_groups, key=self.sortKey)

    def sortKey(self, group_title):
        ids = sorted([compref.id for compref in self.comps[group_title]])
        if ids:
            return ids[0]
        else:
            return ''

    def getSectionAdditionalComps(self):
        section = self.getSection()
        for comp in getAdditionalCompetencies(section):
            self.additionalComps[comp] = None

    def getCoursesAdditionalComps(self):
        courses = self.getCourses()
        for course in courses:
            for comp in getAdditionalCompetencies(course):
                self.additionalComps[comp] = None

    def getSection(self):
        section = removeSecurityProxy(self.context.__parent__).__parent__
        return removeSecurityProxy(section)

    def getCourses(self):
        if ICourse.providedBy(self.context.__parent__.__parent__):
            return [removeSecurityProxy(self.context.__parent__.__parent__),]
        return list(self.getSection().courses)

    def isAdditionalCompetency(self, comp):
        return comp in self.additionalComps

    def getCompIdClass(self, comp):
        if self.isAdditionalCompetency(comp):
            return "additional-comp"
        return "%s-comp" % comp.optionalFlag


class SkillDriversView(ContainerView):

    index_title = u'Assignments:'

    def allowCourseDelete(self):
        return True

    def delete_disabled(self, skilldriver):
        return False

    def getParent (self, skilldriver):
        return None


class SectionSkillDriversView(ContainerView):

    index_title = u'Assignments for this Section'

    def allowCourseDelete(self):
        return False

    def canModify(self):
        return True

    def delete_disabled(self, skilldriver):
        skilldriver = removeSecurityProxy(skilldriver)
        dependable = IDependable(skilldriver, None)
        if dependable is not None and bool(dependable.dependents()):
            return True
        else:
            if self.getCourse(skilldriver):
                return True
            else:
                return False

    def getCourse(self, skilldriver):
        skilldriver = removeSecurityProxy(skilldriver)
        if type(skilldriver.__getstate__()['__parent__'].__parent__) == Course:
            return skilldriver.__getstate__()['__parent__'].__parent__
        else:
            return False


class SkillDriverSelectedCompetenciesFormView(SelectedCompetenciesFormView):
    """Form for managing selected competencies in relation to skill
    drivers.
    """

    def handle_link_action(self, action, data):
        sessionData = ISession(self.request)['cando.virginia.competency']
        selected = sessionData.get('selected',[])
        for comp in selected:
            assignAdditionalCompetencies(comp, removeSecurityProxy(self.context))
        self.status = "Selected Competencies Placed in Assignments"


class SkillDriverViewlet(PersonDashboardCategory):
    """A viewlet showing data dealing with skilldrivers"""

    def instructorOf(self):
        """Get the courses/sections an instructor teaches"""
        return getRelatedObjects(self.context, relationships.URISection)

    def instructorSections(self):
        """Return all sections an instructor can grade"""
        results = []
        for section in self.instructorOf():
             results.append({'object':section})
        return results

    def driversByClass(self,some_section):
       """This gets all the skilldrivers in a section when given a section"""
       sectSkillDrivers = ISectionSkillDrivers(some_section)
       allSkillDriversInSection = sectSkillDrivers.values()
       return allSkillDriversInSection
