#
# CanDo - Web based competency tracking system.
# Copyright (C) 2006 Arlington Public Schools, Arlington, VA and The
# Shuttleworth Foundation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Level-related Tests

$Id$
"""
__docformat__='restructuredtext'

import unittest
from zope.component import provideAdapter
from zope.event import notify
from zope.app.container.contained import ObjectAddedEvent

from schooltool.app.interfaces import ISchoolToolApplication
from schooltool.app.app import getSchoolToolApplication
from schooltool.course.interfaces import ICourse, ISection

from cando.skilldriver.interfaces import ISkillDriver
from cando.virginia import competency
from cando.virginia.interfaces import ICompetencies, ICompetencyList
from cando.virginia.interfaces import ICompetency, ICompetencyReference


def buildSampleCompetencies():
    app = getSchoolToolApplication()
    compRefTree = ICompetencies(app)
    globalCompetencies = competency.getAppGlobalCompetencies(app)

    # build dictionary of global ids and their descriptions.
    competencies = [
        ('http://domain.org/1', u'Algebra11'),
        ('http://domain.org/2', u'Algebra12'),
        ('http://domain.org/3', u'Algebra21'),
        ('http://domain.org/4', u'Algebra22'),
        ('http://domain.org/5', u'Geometry11'),
        ('http://domain.org/6', u'Geometry12'),
        ('http://domain.org/7', u'Geometry21'),
        ('http://domain.org/8', u'Geometry22'),
        ('http://domain.org/9', u'Chemistry11'),
        ('http://domain.org/10', u'Chemistry12'),
        ('http://domain.org/11', u'Chemistry21'),
        ('http://domain.org/12', u'Chemistry22'),
        ('http://domain.org/13', u'Physics11'),
        ('http://domain.org/14', u'Physics12'),
        ('http://domain.org/15', u'Physics21'),
        ('http://domain.org/16', u'Physics22'),
        ('http://domain.org/17', u'Chemistry31'),
        ]
    for globalId, title in competencies:
        globalCompetencies[globalId] = competency.Competency(title)

    # build competency tree with two areas, 'Math', and 'Science'.
    # 'Math' will have courses 'Algebra' and 'Geometry'.
    # 'Science' will have courses 'Chemistry' and 'Physics'.
    # each course will have two course comp groups, each having two comps.
    # Note that for algebra group2 we added '  more'.  This is to test
    # handling the fact that browsers do not always send double spaces to
    # the server.
    # We added a new group for the Chemistry course with one comp in it
    math = compRefTree['Math'] = competency.Competencies(u'Math')
    science = compRefTree['Science'] = competency.Competencies(u'Science')
    algebra = math['Algebra'] = competency.Competencies(u'Algebra')
    geometry = math['Geometry'] = competency.Competencies(u'Geometry')
    chemistry = science['Chemistry'] = competency.Competencies(u'Chemistry')
    physics = science['Physics'] = competency.Competencies(u'Physics')
    algebra['group1'] = competency.Competencies(u'Group1')
    algebra['group2'] = competency.Competencies(u'Group2  more')
    geometry['group1'] = competency.Competencies(u'Group1')
    geometry['group2'] = competency.Competencies(u'Group2')
    chemistry['group3'] = competency.Competencies(u'Group3')
    chemistry['group1'] = competency.Competencies(u'Group1')
    chemistry['group2'] = competency.Competencies(u'Group2')
    physics['group1'] = competency.Competencies(u'Group1')
    physics['group2'] = competency.Competencies(u'Group2')

    # when we put our two competencies in each group, note that the global ids
    # are unique across all competencies whereas the ids are only unique across
    # the course competencies.
    algebra['group1']['comp1'] = competency.CompetencyReference(
        u'001', 'http://domain.org/1')
    notify(ObjectAddedEvent(algebra['group1']['comp1']))
    algebra['group1']['comp2'] = competency.CompetencyReference(
        u'002', 'http://domain.org/2')
    notify(ObjectAddedEvent(algebra['group1']['comp2']))
    algebra['group2']['comp1'] = competency.CompetencyReference(
        u'003', 'http://domain.org/3')
    notify(ObjectAddedEvent(algebra['group2']['comp1']))
    algebra['group2']['comp2'] = competency.CompetencyReference(
        u'004', 'http://domain.org/4')
    notify(ObjectAddedEvent(algebra['group2']['comp2']))

    geometry['group1']['comp1'] = competency.CompetencyReference(
        u'005', 'http://domain.org/5')
    notify(ObjectAddedEvent(geometry['group1']['comp1']))
    geometry['group1']['comp2'] = competency.CompetencyReference(
        u'006', 'http://domain.org/6')
    notify(ObjectAddedEvent(geometry['group1']['comp2']))
    geometry['group2']['comp1'] = competency.CompetencyReference(
        u'007', 'http://domain.org/7')
    notify(ObjectAddedEvent(geometry['group2']['comp1']))
    geometry['group2']['comp2'] = competency.CompetencyReference(
        u'008', 'http://domain.org/8')
    notify(ObjectAddedEvent(geometry['group2']['comp2']))

    chemistry['group1']['comp1'] = competency.CompetencyReference(
        u'009', 'http://domain.org/9')
    notify(ObjectAddedEvent(chemistry['group1']['comp1']))
    chemistry['group1']['comp2'] = competency.CompetencyReference(
        u'010', 'http://domain.org/10')
    notify(ObjectAddedEvent(chemistry['group1']['comp2']))
    chemistry['group2']['comp1'] = competency.CompetencyReference(
        u'011', 'http://domain.org/11')
    notify(ObjectAddedEvent(chemistry['group2']['comp1']))
    chemistry['group2']['comp2'] = competency.CompetencyReference(
        u'012', 'http://domain.org/12')
    notify(ObjectAddedEvent(chemistry['group2']['comp2']))
    chemistry['group3']['comp1'] = competency.CompetencyReference(
        u'017', 'http://domain.org/17')
    notify(ObjectAddedEvent(chemistry['group3']['comp1']))

    physics['group1']['comp1'] = competency.CompetencyReference(
        u'013', 'http://domain.org/13')
    notify(ObjectAddedEvent(physics['group1']['comp1']))
    physics['group1']['comp2'] = competency.CompetencyReference(
        u'014', 'http://domain.org/14')
    notify(ObjectAddedEvent(physics['group1']['comp2']))
    physics['group2']['comp1'] = competency.CompetencyReference(
        u'015', 'http://domain.org/15')
    notify(ObjectAddedEvent(physics['group2']['comp1']))
    physics['group2']['comp2'] = competency.CompetencyReference(
        u'016', 'http://domain.org/16')
    notify(ObjectAddedEvent(physics['group2']['comp2']))


def setUpGlobalComps():
    # initialize the global competency tree, catalog and index
    # register the catalog lookup components
    app = getSchoolToolApplication()
    init = competency.CompetencyInit(app)
    init()
    provideAdapter(competency.getGlobalIdUtility)

    # register the adapter that adapts the app to the global comps
    provideAdapter(competency.getAppCompetencyRefTree,
                   adapts=[ISchoolToolApplication],
                   provides=ICompetencies)

    provideAdapter(competency.getAppCompetencyRefTree,
                   (ISchoolToolApplication,),
                   ICompetencies)
    provideAdapter(competency.listAssignedCompetencies,
                   (ICourse,),
                   ICompetencyList)
    provideAdapter(competency.listAssignedCompetencies,
                   (ISkillDriver,),
                   ICompetencyList)
    provideAdapter(competency.listSectionCompetencies,
                   (ISection,),
                   ICompetencyList)
    provideAdapter(competency.getCompetencyFromCompRef,
                   (ICompetencyReference,),
                   ICompetency)

    buildSampleCompetencies()


def test_suite():
    return unittest.TestSuite()
