from datetime import datetime
from lxml import etree
from persistent.list import PersistentList
import transaction

from zope.app.container.interfaces import INameChooser
from zope.app.generations.utility import findObjectsProviding
from zope.copypastemove.interfaces import IObjectMover
from zope.interface import implements
from zope.security.proxy import removeSecurityProxy

from schooltool.app.app import getSchoolToolApplication
from cando.virginia import interfaces
from cando.virginia.competency import Competencies
from cando.virginia.competency import Competency, CompetencyReference
from cando.virginia.competency import getAppGlobalCompetencies
from interfaces import ICompetencyXMLImporter


HOLDS_LISTS = 'lists'
HOLDS_COMPS = 'comps'


def removeNewlines(instr):
    if instr is None:
        return None
    newstr = instr.strip().replace('\n', ' ')
    newstr = newstr.replace('\r', ' ')
    return newstr


class XMLImportError(Exception):
    """An error ocurred when XML was being imported"""

    def __init__(self, message):
        self.message = message


class TempCompetencies(object):
    """It represents a ICompetencies object. It can contain
    TempCompetencyReferences or other TempCompetencies
    """

    def __init__(self):
        self.id = ''
        self.description = ''
        self.list = []
        self.type = ''

    def setId(self, tid):
    	self.id = removeNewlines(tid)

    def setDescription(self, des):
        self.description = removeNewlines(des)

    def setType(self, typ):
        """Must be called to let this object have any meaning"""
    	self.type = removeNewlines(typ)

    def addComp(self, cmp):
        self.list.append(cmp)

    def addList(self, lst):
        self.list.append(lst)


class TempCompetencyReference(object):
    """It represents a ICompetencyReference object."""

    def __init__(self):
        self.globalId = ''
        self.id = ''
        self.title = ''
        self.flag = ''
        self.sol_correlations = {}

    def setGlobalId(self, gid):
        self.globalId = removeNewlines(gid)

    def setId(self, tid):
        self.id = removeNewlines(tid)

    def setTitle(self, tle):
        self.title = removeNewlines(tle)

    def setFlag(self, flg):
        self.flag = removeNewlines(flg)

    def setSOLs(self, sol_string):
        self.sol_correlations = {}
        if not sol_string:
            return
        categories = sol_string.split('|')
        for cat_string in categories:
            idx = cat_string.find(':')
            ids = [unicode(s.strip()) for s in cat_string[idx+1:].split(',')]
            if idx >= 0:
                cat_name = unicode(cat_string[:idx])
            else:
                # No category separator found, asume empty category name
                cat_name = u''
            self.sol_correlations[cat_name] = ids


class CompetencyXMLImporter(object):
    """XML Import for competency list.

    Check the importing_rules.txt file."""

    implements(ICompetencyXMLImporter)

    def __init__(self, context):
        self.xmlEtree = None
        self.rootList = TempCompetencies()
        self.context = context
        self.x = ''
        self.errors = []

    def findRealCompRefs(self, container):
        return findObjectsProviding(container, interfaces.ICompetencyReference)

    def isAppCompetencyRefTree(self, container):
        app = getSchoolToolApplication()
        return container is interfaces.ICompetencies(app)

    def realProgramAreas(self):
        return dict([(area.title, area) for area in self.context.values()])

    def realCourses(self):
        result = {}
        for realProgramArea in self.realProgramAreas().values():
            for course in realProgramArea.values():
                course_data = self.parseCourseTitle(course.title)
                if course_data['course_number']:
                    result[course_data['course_number']] = course
                else:
                    result[course_data['course_name']] = course
        return result

    def realCompGroups(self, course):
        return dict([(group.title, group)
                     for group in course.values()])

    def realCompRefsByDescription(self, course):
        result = {}
        for realCompRef in self.findRealCompRefs(course):
            result[realCompRef.competency.title] = realCompRef
        return result

    def realCompRefsByGlobalId(self, course):
        result = {}
        for realCompRef in self.findRealCompRefs(course):
            result[realCompRef.globalId] = realCompRef
        return result

    def isNewProgramArea(self, tempProgramArea):
        return tempProgramArea.description not in self.realProgramAreas()

    def isNewCourse(self, tempCourse):
        course_data = self.parseCourseTitle(tempCourse.description)
        if course_data['course_number']:
            return course_data['course_number'] in self.realCourses()
        else:
            return course_data['course_name'] in self.realCourses()

    def createRealProgramArea(self, tempProgramArea):
        result = Competencies(tempProgramArea.description)
        chooser = INameChooser(self.context)
        name = chooser.chooseName('', result)
        self.context[name] = result
        return result

    # XXX: The importer should be able to import individual program
    #      areas or courses or groups, like the old one did
    def importFromXML(self, xmlin):
        try:
            if not self.isAppCompetencyRefTree(self.context):
                raise XMLImportError('Please use this option from the main '
                                     'Competency reference tree')
            xmlin = xmlin.replace('\c\r', '\n')
            xmlin = xmlin.strip()
            if xmlin.startswith("<?xml version="):
                try:
                    self.xmlEtree = etree.XML(xmlin)
                except (etree.XMLSyntaxError, ValueError), strerror:
                    raise XMLImportError('Invalid xml syntax: %s' % (strerror,))
            else:
                raise XMLImportError('Not a valid xml file')
            for stuff in self.xmlEtree.nsmap:
                xmlnskey = self.xmlEtree.nsmap[stuff]
            self.x = '{' + xmlnskey + '}'
            result = self.parse(self.xmlEtree, self.rootList)
            if not result:
                return False
            self.processXMLTree()
        except (XMLImportError,), error:
            self.errors.append(error.message)
            transaction.abort()
            return False
        else:
            return True

    def processXMLTree(self):
        app = getSchoolToolApplication()
        globalCompetencies = getAppGlobalCompetencies(app)
        tempCourses = []
        for tempProgramArea in self.rootList.list:
            self.validateTempProgramArea(tempProgramArea)
            if self.isNewProgramArea(tempProgramArea):
                realProgramArea = self.createRealProgramArea(tempProgramArea)
            for tempCourse in tempProgramArea.list:
                self.validateTempCourse(tempCourse)
                tempCourses.append((tempProgramArea, tempCourse,))
        for tempProgramArea, tempCourse in tempCourses:
            tempCourseNumber = self.tempCourseNumber(tempCourse)
            if tempCourseNumber in self.realCourses():
                realCourse = self.realCourses()[tempCourseNumber]
                realCourse.title = unicode(tempCourse.description)
                programArea = self.realProgramAreas()[tempProgramArea.description]
                if realCourse.__parent__ is not programArea:
                    # Course has been moved to a new ProgramArea
                    # Welsh said we shouldn't take care of this
                    pass
                tempCompRefs = []
                for tempCompGroup in tempCourse.list:
                    self.validateTempCompGroup(tempCompGroup)
                    if tempCompGroup.description not in self.realCompGroups(realCourse):
                        newRealCompGroup = Competencies(unicode(tempCompGroup.description))
                        chooser = INameChooser(realCourse)
                        name = chooser.chooseName('', newRealCompGroup)
                        realCourse[name] = newRealCompGroup
                    for tempCompRef in tempCompGroup.list:
                        self.validateTempCompRef(tempCompRef)
                        tempCompRefs.append((tempCompGroup, tempCompRef,))
                for tempCompGroup, tempCompRef in tempCompRefs:
                    if tempCompRef.globalId in globalCompetencies:
                        realCompetency = globalCompetencies[tempCompRef.globalId]
                        realCompRef = self.realCompRefsByGlobalId(realCourse).get(tempCompRef.globalId)
                        if realCompRef is None:
                            # realCompetency exists but the
                            # tempCompRef is new for this realCourse
                            realCompRef = self.realCompRefsByDescription(realCourse).get(tempCompRef.title)
                            if realCompRef is None:
                                self.validateTempCompRef(tempCompRef)
                                realCompRef = self.createRealCompRef(tempCompRef)
                        if tempCompRef.title != realCompetency.title:
                            if realCompRef.modified is None:
                                realCompRef.modified = PersistentList()
                            realCompRef.modified.append((datetime.now(),
                                                         realCompetency.title,
                                                         unicode(tempCompRef.title)))
                            realCompetency.title = unicode(tempCompRef.title)
                        realCompRef.id = tempCompRef.id
                        realCompRef.optionalFlag = tempCompRef.flag
                        realCompRef.setSOLCorrelations(tempCompRef.sol_correlations)
                        realCompRef.deprecated = False
                        realCompGroup = realCompRef.__parent__
                        if realCompGroup is None:
                            # New Group for this CompRef
                            realCompGroup = self.realCompGroups(realCourse).get(tempCompGroup.description)
                            if realCompGroup is None:
                                # New Group in this RealCourse
                                self.validateTempCompGroup(tempCompGroup)
                                description = unicode(tempCompGroup.description)
                                realCompGroup = Competencies(description)
                                chooser = INameChooser(realCourse)
                                name = chooser.chooseName('', realCompGroup)
                                realCourse[name] = realCompGroup
                            chooser = INameChooser(realCompGroup)
                            name = chooser.chooseName(realCompRef.id,
                                                      realCompRef)
                            realCompGroup[name] = realCompRef
                        if realCompGroup.title != tempCompGroup.description:
                            mover = IObjectMover(realCompRef)
                            mover.moveTo(self.realCompGroups(realCourse)[tempCompGroup.description])
                    else:
                        description = tempCompRef.title
                        if description in self.realCompRefsByDescription(realCourse):
                            # an incorrectly changed global ID
                            realCompRef = self.realCompRefsByDescription(realCourse)[description]
                            realCompRef.id = tempCompRef.id
                            realCompRef.optionalFlag = tempCompRef.flag
                            realCompRef.setSOLCorrelations(tempCompRef.sol_correlations)
                            realCompRef.deprecated = False
                            realCompGroup = realCompRef.__parent__
                            if realCompGroup.title != tempCompGroup.description:
                                mover = IObjectMover(realCompRef)
                                mover.moveTo(self.realCompGroups(realCourse)[tempCompGroup.description])
                        else:
                            # new competency reference
                            newRealCompRef = self.createRealCompRef(tempCompRef)
                            realCompGroup = self.realCompGroups(realCourse)[tempCompGroup.description]
                            chooser = INameChooser(realCompGroup)
                            name = chooser.chooseName(newRealCompRef.id,
                                                      newRealCompRef)
                            realCompGroup[name] = newRealCompRef
                for realCompGroupName in realCourse.keys():
                    if not len(realCourse[realCompGroupName]):
                        del realCourse[realCompGroupName]
            else:
                description = tempProgramArea.description
                realProgramArea = self.realProgramAreas()[description]
                newRealCourse = Competencies(unicode(tempCourse.description))
                chooser = INameChooser(realProgramArea)
                name = chooser.chooseName('', newRealCourse)
                realProgramArea[name] = newRealCourse
                for tempCompGroup in tempCourse.list:
                    self.validateTempCompGroup(tempCompGroup)
                    description = unicode(tempCompGroup.description)
                    newRealCompGroup = Competencies(description)
                    chooser = INameChooser(newRealCourse)
                    name = chooser.chooseName('', newRealCompGroup)
                    newRealCourse[name] = newRealCompGroup
                    for tempCompRef in tempCompGroup.list:
                        self.validateTempCompRef(tempCompRef)
                        newRealCompRef = self.createRealCompRef(tempCompRef)
                        chooser = INameChooser(newRealCompGroup)
                        name = chooser.chooseName(newRealCompRef.id,
                                                  newRealCompRef)
                        newRealCompGroup[name] = newRealCompRef
                for realCompGroupName in newRealCourse.keys():
                    if not len(newRealCourse[realCompGroupName]):
                        del newRealCourse[realCompGroupName]
        return True

    def parse(self, relativeRoot, relativeList):
        """ for every complist, add the list and recurse on it. Otherwise,
        add the comp. relativeRoot is the current position in the
        document tree and is used as a starting point for recursion."""
        try:
            test = relativeRoot.getchildren()
        except etree.XMLSyntaxError, strerror:
            raise XMLImportError('Invalid xml syntax: %s' % (strerror,))
        for child in relativeRoot.getchildren():
            if child.tag == self.x+'id':
                relativeList.setId(child.text)
            elif child.tag == self.x+'description':
                relativeList.setDescription(child.text)
            elif child.tag == self.x+'complist':
                if relativeList.type == '':
                    relativeList.type = HOLDS_LISTS
                if relativeList.type != HOLDS_LISTS: #check for incorrect input
                    raise XMLImportError('Invalid xml data: line %s: '
                                         'Unexpected tag: %s' %
                                         (child.sourceline,
                                          child.tag.replace(self.x, ''))
                                         )
                tempSubList = TempCompetencies()
                tempSubList.line = child.sourceline
                if not self.parse(child, tempSubList):
                    return False
                relativeList.addList(tempSubList)
            elif child.tag == self.x+'comp':
                if relativeList.type == '':
                    relativeList.type = HOLDS_COMPS
                if relativeList.type != HOLDS_COMPS:
                    raise XMLImportError('Invalid xml data: line %s: '
                                         'Unexpected tag: %s' %
                                         (child.sourceline,
                                          child.tag.replace(self.x, ''))
                                         )
                    return False
                tempComp = TempCompetencyReference()
                tempComp.line = child.sourceline
                if self.parseCompData(child, tempComp):
                    relativeList.addComp(tempComp)
                else:
                    return False
            else:
                if type(child) != etree._Comment:
                    error = child.tag.replace(self.x, '')
                    line = child.sourceline
                    if error in ('id', 'description', 'complist', 'comp'):
                        raise XMLImportError('Invalid xml data: line %s: '
                                             'Needs proper namespace: %s' %
                                             (line, self.x)
                                             )
                    else:
                        raise XMLImportError('Invalid xml data: line %s: '
                                             'Unexpected tag: %s' %
                                             (line, error)
                                             )
        return True

    def parseCompData(self, relativeRoot, tempComp):
        for child in relativeRoot.getchildren():
            if child.tag == self.x+'global-id':
                tempComp.setGlobalId(child.text)
            elif child.tag == self.x+'id':
                tempComp.setId(child.text)
            elif child.tag == self.x+'title':
                tempComp.setTitle(child.text)
            elif child.tag == self.x+'flag':
                tempComp.setFlag(child.text)
            elif child.tag == self.x+'sols':
                tempComp.setSOLs(child.text)
            else:
                if type(child) != etree._Comment:
                    error = child.tag.replace(self.x, '')
                    line = child.sourceline
                    if error in ('id', 'global-id', 'title', 'flag'):
                        raise XMLImportError('Invalid xml data: line %s: '
                                             'Needs proper namespace: %s' %
                                             (line, self.x)
                                             )
                    else:
                        raise XMLImportError('Invalid xml data: line %s: '
                                             'Unexpected tag: %s' %
                                             (line, error)
                                             )
        return True

    def parseCourseTitle(self, title):
        result = {}
        try:
            result['course_name'] = title.split(' - ')[0].strip()
        except (IndexError,):
            result['course_name'] = ''
        try:
            result['course_number'] = title.split(' - ')[1].strip()
        except (IndexError,):
            result['course_number'] = ''
        try:
            result['course_duration'] = title.split(' - ')[2].strip()
        except (IndexError,):
            result['course_duration'] = ''
        return result

    def hasNoDescription(self, obj):
        return not obj.description

    def hasNoTitle(self, obj):
        return not obj.title

    def hasNoId(self, obj):
        return not obj.id

    def hasNoFlag(self, obj):
        return not obj.flag

    def hasNoGlobalId(self, obj):
        return not obj.globalId

    def tempCourseNumber(self, tempCourse):
        course_data = self.parseCourseTitle(tempCourse.description)
        return course_data['course_number'] or course_data['course_name']

    def createRealCompRef(self, tempCompRef):
        title = unicode(tempCompRef.title)
        globalId = unicode(tempCompRef.globalId)
        globalCompetencies = getAppGlobalCompetencies(getSchoolToolApplication())
        if globalId not in globalCompetencies: # new globalId
            old_title = title
            globalCompetencies[globalId] = Competency(title)
        else:
            old_title = globalCompetencies[globalId].title
        globalCompetencies[globalId].title = title
        result = CompetencyReference(unicode(tempCompRef.id), globalId,
                                      optionalFlag=tempCompRef.flag)
        if old_title != title:
            if result.modified is None:
                result.modified = PersistentList()
            result.modified.append((datetime.now(), old_title, title))
        result.setSOLCorrelations(tempCompRef.sol_correlations)
        return result

    def addComp(self, container, comp, suggestedName):
        container = removeSecurityProxy(container)
        for existingComp in container.values():
            if existingComp.title == comp.title:
                if interfaces.ICompetencyReference.providedBy(existingComp):
                    existingComp.id = comp.id
                    existingComp.optionalFlag = comp.optionalFlag
                    existingComp.globalId = comp.globalId
                    if comp.modified:
                        if existingComp.modified is None:
                            existingComp.modified = PersistentList()
                        existingComp.modified.extend(comp.modified)
                return existingComp
        else:
            chooser = INameChooser(container)
            name = chooser.chooseName(suggestedName, comp)
            container[name] = comp
            return comp

    def validateTempProgramArea(self, tempProgramArea):
        if self.hasNoDescription(tempProgramArea):
            raise XMLImportError('Invalid xml data: line %s: '
                                 'complist is missing description' %
                                 (tempProgramArea.line,))

    def validateTempCourse(self, tempCourse):
        if self.hasNoDescription(tempCourse):
            raise XMLImportError('Invalid xml data: line %s: '
                                 'complist is missing description' %
                                 (tempCourse.line,))

    def validateTempCompGroup(self, tempCompGroup):
        if self.hasNoDescription(tempCompGroup):
            raise XMLImportError('Invalid xml data: line %s: '
                                 'complist is missing description' %
                                 (tempCompGroup.line,))

    def validateTempCompRef(self, tempCompRef):
        NO_TITLE = 'Invalid xml data: line %s: competency is missing title'
        NO_ID = 'Invalid xml data: line %s: competency is missing id'
        NO_FLAG = 'Invalid xml data: line %s: competency is missing flag'
        NO_GLOBAL_ID = 'Invalid xml data: line %s: competency is missing ' \
                       'global-id'
        message = None
        if self.hasNoTitle(tempCompRef):
            message = NO_TITLE
        if self.hasNoId(tempCompRef):
            message = NO_ID
        if self.hasNoFlag(tempCompRef):
            message = NO_FLAG
        if self.hasNoGlobalId(tempCompRef):
            message = NO_GLOBAL_ID
        if message is not None:
            raise XMLImportError(message % (tempCompRef.line,))
